@@ -13,6 +13,7 @@ from utils import (mock_get, open_legacy_httpbin, prepare_mock_browser,
                    setup_mock_browser)
 
 import mechanicalsoup
+import requests
 
 
 def test_request_forward():
@@ -425,6 +426,30 @@ def test_select_form_tag_object():
             browser.select_form(soup.find("p"))
 
 
+def test_select_form_associated_elements():
+    """Test associated elements outside the form tag"""
+    forms = """<form id="a"><input><textarea></form><input form="a">
+               <textarea form="a"/><input form="b">
+               <form id="ab" action="/test.php"><input></form>
+               <textarea form="ab"></textarea>
+            """
+    with mechanicalsoup.StatefulBrowser() as browser:
+        browser.open_fake_page(forms)
+        elements_form_a = set([
+            "<input/>", "<textarea></textarea>",
+            '<input form="a"/>', '<textarea form="a"></textarea>'])
+        elements_form_ab = set(["<input/>", '<textarea form="ab"></textarea>'])
+        form_by_str = browser.select_form("#a")
+        form_by_tag = browser.select_form(browser.page.find("form", id='a'))
+        form_by_css = browser.select_form("form[action$='.php']")
+        assert set([str(element) for element in form_by_str.form.find_all((
+            "input", "textarea"))]) == elements_form_a
+        assert set([str(element) for element in form_by_tag.form.find_all((
+            "input", "textarea"))]) == elements_form_a
+        assert set([str(element) for element in form_by_css.form.find_all((
+            "input", "textarea"))]) == elements_form_ab
+
+
 def test_referer_follow_link(httpbin):
     browser = mechanicalsoup.StatefulBrowser()
     open_legacy_httpbin(browser, httpbin)
@@ -499,7 +524,7 @@ def test_referer_submit_headers(httpbin):
 
 @pytest.mark.parametrize('expected, kwargs', [
     pytest.param('/foo', {}, id='none'),
-    pytest.param('/get', {'text': 'Link'}, id='text'),
+    pytest.param('/get', {'string': 'Link'}, id='string'),
     pytest.param('/get', {'url_regex': 'get'}, id='regex'),
 ])
 def test_follow_link_arg(httpbin, expected, kwargs):
@@ -530,7 +555,7 @@ def test_follow_link_ua(httpbin):
     browser = mechanicalsoup.StatefulBrowser()
     # html = '<a href="/foo">Bar</a><a href="/get">Link</a>'
     # browser.open_fake_page(html, httpbin.url)
-    browser.open(httpbin.url)
+    open_legacy_httpbin(browser, httpbin)
     bs4_kwargs = {'url_regex': 'user-agent'}
     requests_kwargs = {'headers': {"User-Agent": '007'}}
     resp = browser.follow_link(bs4_kwargs=bs4_kwargs,
@@ -779,5 +804,15 @@ def test_refresh_error():
         browser.refresh()
 
 
+def test_requests_session_and_cookies(httpbin):
+    """Check that the session object passed to the constructor of
+    StatefulBrowser is actually taken into account."""
+    s = requests.Session()
+    requests.utils.add_dict_to_cookiejar(s.cookies, {'key1': 'val1'})
+    browser = mechanicalsoup.StatefulBrowser(session=s)
+    resp = browser.get(httpbin + "/cookies")
+    assert resp.json() == {'cookies': {'key1': 'val1'}}
+
+
 if __name__ == '__main__':
     pytest.main(sys.argv)
